@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 1990-2002 Info-ZIP.  All rights reserved.
+  Copyright (c) 1990-2005 Info-ZIP.  All rights reserved.
 
   See the accompanying file LICENSE, version 2000-Apr-09 or later
   (the contents of which are also included in unzip.h) for terms of use.
@@ -47,6 +47,11 @@
 #  endif
 #endif
 
+#if (defined(USE_ZLIB) && !defined(NO_DEFLATE64))
+   /* zlib does not (yet?) provide Deflate64(tm) support */
+#  define NO_DEFLATE64
+#endif
+
 #ifdef NO_DEFLATE64
    /* disable support for Deflate64(tm) */
 #  ifdef USE_DEFLATE64
@@ -87,6 +92,24 @@
     OS-dependent configuration for UnZip internals
   ---------------------------------------------------------------------------*/
 
+/* Some compiler distributions for Win32/i386 systems try to emulate
+ * a Unix (POSIX-compatible) environment.
+ */
+#if (defined(WIN32) && defined(UNIX))
+   /* UnZip does not support merging both ports in a single executable. */
+#  if (defined(FORCE_WIN32_OVER_UNIX) && defined(FORCE_UNIX_OVER_WIN32))
+     /* conflicting choice requests -> we prefer the Win32 environment */
+#    undef FORCE_UNIX_OVER_WIN32
+#  endif
+#  ifdef FORCE_WIN32_OVER_UNIX
+     /* native Win32 support was explicitely requested... */
+#    undef UNIX
+#  else
+     /* use the POSIX (Unix) emulation features by default... */
+#    undef WIN32
+#  endif
+#endif
+
 /* bad or (occasionally?) missing stddef.h: */
 #if (defined(M_XENIX) || defined(DNIX))
 #  define NO_STDDEF_H
@@ -166,7 +189,10 @@
  * filesystem or `U:' drive under Atari MiNT.)  Watcom C was previously
  * included on this list; it would be good to know what version the problem
  * was fixed at, if it did exist.  */
-#if (defined(__TURBOC__) || defined(VMS) || defined(__MINT__))
+#if (defined(__TURBOC__) && !defined(WIN32))
+/*#  define WILD_STAT_BUG*/
+#endif
+#if (defined(VMS) || defined(__MINT__))
 #  define WILD_STAT_BUG
 #endif
 
@@ -175,9 +201,13 @@
   ---------------------------------------------------------------------------*/
 
 #ifdef EFT
-#  define LONGINT off_t  /* Amdahl UTS nonsense ("extended file types") */
+#  define Z_OFF_T off_t  /* Amdahl UTS nonsense ("extended file types") */
 #else
-#  define LONGINT long
+#if (defined(UNIX) && defined(_FILE_OFFSET_BITS) && (_FILE_OFFSET_BITS == 64))
+#  define Z_OFF_T off_t /* 64bit offsets to support 2GB < zipfile size < 4GB */
+#else
+#  define Z_OFF_T long
+#endif
 #endif
 
 #ifdef MODERN
@@ -190,8 +220,8 @@
    typedef size_t extent;
 #else /* !MODERN */
 #  ifndef AOS_VS         /* mostly modern? */
-     LONGINT lseek();
-#    ifdef VAXC          /* not fully modern, but does have stdlib.h and void */
+     Z_OFF_T lseek();
+#    ifdef VAXC          /* not fully modern, but has stdlib.h and void */
 #      include <stdlib.h>
 #    else
        char *malloc();
@@ -288,36 +318,20 @@
 #  endif
 #endif
 
+/*---------------------------------------------------------------------------
+    AtheOS section:
+  ---------------------------------------------------------------------------*/
+
+#ifdef __ATHEOS__
+#  include "atheos/athcfg.h"
+#endif
+
 /*---------------------------------------------------------------------------
     BeOS section:
   ---------------------------------------------------------------------------*/
 
 #ifdef __BEOS__
-#  include <sys/types.h>          /* [cjh]:  This is pretty much a generic  */
-#  include <sys/stat.h>           /* POSIX 1003.1 system; see beos/ for     */
-#  include <fcntl.h>              /* extra code to deal with our extra file */
-#  include <sys/param.h>          /* attributes. */
-#  include <unistd.h>
-#  include <utime.h>
-#  define DIRENT
-#  include <time.h>
-#  ifndef DATE_FORMAT
-#    define DATE_FORMAT DF_MDY  /* GRR:  customize with locale.h somehow? */
-#  endif
-#  define lenEOL        1
-#  define PutNativeEOL  *q++ = native(LF);
-#  define SCREENSIZE(ttrows, ttcols)  screensize(ttrows, ttcols)
-#  define SCREENWIDTH 80
-#  define USE_EF_UT_TIME
-#  define SET_DIR_ATTRIB
-#  if (!defined(NOTIMESTAMP) && !defined(TIMESTAMP))
-#    define TIMESTAMP
-#  endif
-#  define RESTORE_UIDGID
-#  define NO_STRNICMP             /* not in the x86 headers at least */
-#  define INT_SPRINTF
-#  define SYMLINKS
-#  define MAIN main_stub          /* now that we're using a wrapper... */
+#  include "beos/beocfg.h"
 #endif
 
 /*---------------------------------------------------------------------------
@@ -428,7 +442,9 @@
 #  define DATE_FORMAT     dateformat()
 #  define lenEOL          2
 #  define PutNativeEOL    {*q++ = native(CR); *q++ = native(LF);}
-#  define USE_EF_UT_TIME
+#  if (!defined(NO_EF_UT_TIME) && !defined(USE_EF_UT_TIME))
+#    define USE_EF_UT_TIME
+#  endif
 #endif /* MSDOS || OS2 || FLEXOS */
 
 /*---------------------------------------------------------------------------
@@ -498,8 +514,16 @@
 #ifdef TANDEM
 #  include "tandem.h"
 #  include <fcntl.h>
-#  if (defined(__INT32) && !defined(INT_16BIT))
-#    define INT_16BIT   /* report "int" size is 16-bit to inflate setup */
+#  ifndef __INT32
+     /* We are compiling with non-WIDE memory model, int = 16 bits */
+#    ifndef INT_16BIT
+#      define INT_16BIT   /* report "int" size is 16-bit to inflate setup */
+#    endif
+#    ifdef USE_DEFLATE64
+       /* Following required for 64k WSIZE of Deflate64 support */
+#      define MED_MEM     /* else OUTBUFSIZ is 64K and fails in do_string */
+#      define INBUFSIZ  8192  /* but larger buffer for real OSes */
+#    endif
 #  endif
    /* use a single LF delimiter so that writes to 101 text files work */
 #  define PutNativeEOL  *q++ = native(LF);
@@ -623,16 +647,40 @@
 #    define TIMESTAMP
 #  endif
 #  define RESTORE_UIDGID
+#  ifdef __DECC
+     /* File open callback ID values. */
+#    define OPENR_ID 1
+     /* File open callback ID storage. */
+     extern int openr_id;
+     /* File open callback function. */
+     extern int acc_cb();
+     /* Option macros for open().
+      * General: Stream access
+      *
+      * Callback function (DEC C only) sets deq, mbc, mbf, rah, wbh, ...
+      */
+#    define OPNZIP_RMS_ARGS "ctx=stm", "acc", acc_cb, &openr_id
+#  else /* !__DECC */ /* (So, GNU C, VAX C, ...)*/
+#    define OPNZIP_RMS_ARGS "ctx=stm"
+#  endif /* ?__DECC */
 #endif /* VMS */
 
 /*---------------------------------------------------------------------------
     Win32 (Windows 95/NT) section:
   ---------------------------------------------------------------------------*/
 
-#if (defined(WIN32) && !defined(POCKET_UNZIP))
+#if (defined(WIN32) && !defined(POCKET_UNZIP) && !defined(_WIN32_WCE))
 #  include "win32/w32cfg.h"
 #endif
 
+/*---------------------------------------------------------------------------
+    Win32 Windows CE section (when not using POCKET_UNZIP)
+  ---------------------------------------------------------------------------*/
+
+#if (defined(_WIN32_WCE) || defined(POCKET_UNZIP))
+#  include "wince/wcecfg.h"
+#endif
+
 
 
 
@@ -697,12 +745,16 @@
 #  define DOS_T20_VMS
 #endif
 
-#if (defined(__BEOS__) || defined(UNIX))
-#  define BEO_UNX
+#if (defined(__ATHEOS__) || defined(__BEOS__))
+#  define ATH_BEO
 #endif
 
-#if (defined(BEO_UNX) || defined(THEOS))
-#  define BEO_THS_UNX
+#if (defined(ATH_BEO) || defined(UNIX))
+#  define ATH_BEO_UNX
+#endif
+
+#if (defined(ATH_BEO_UNX) || defined(THEOS))
+#  define ATH_BEO_THS_UNX
 #endif
 
 /* clean up with a few defaults */
@@ -712,6 +764,9 @@
 #ifndef DATE_FORMAT
 #  define DATE_FORMAT   DF_MDY  /* defaults to US convention */
 #endif
+#ifndef DATE_SEPCHAR
+#  define DATE_SEPCHAR  '-'
+#endif
 #ifndef CLOSE_INFILE
 #  define CLOSE_INFILE()  close(G.zipfd)
 #endif
@@ -744,7 +799,7 @@
 #endif
 
 
-#if (defined(DOS_FLX_NLM_OS2_W32) || defined(BEO_UNX) || defined(RISCOS))
+#if (defined(DOS_FLX_NLM_OS2_W32) || defined(ATH_BEO_UNX) || defined(RISCOS))
 #  ifndef HAVE_UNLINK
 #    define HAVE_UNLINK
 #  endif
@@ -836,14 +891,6 @@
 #  endif                    /*  at least 32K for zip's deflate method */
 #endif
 
-#ifndef INBUFSIZ
-#  if (defined(MED_MEM) || defined(SMALL_MEM))
-#    define INBUFSIZ  2048  /* works for MS-DOS small model */
-#  else
-#    define INBUFSIZ  8192  /* larger buffers for real OSes */
-#  endif
-#endif
-
 #ifdef __16BIT__
 #  ifndef INT_16BIT
 #    define INT_16BIT       /* on 16-bit systems int size is 16 bits */
@@ -879,6 +926,27 @@
 #  undef ASM_CRC
 #endif
 
+#ifndef INBUFSIZ
+#  if (defined(MED_MEM) || defined(SMALL_MEM))
+#    define INBUFSIZ  2048  /* works for MS-DOS small model */
+#  else
+#    define INBUFSIZ  8192  /* larger buffers for real OSes */
+#  endif
+#endif
+
+#if (defined(INT_16BIT) && (defined(USE_DEFLATE64) || lenEOL > 1))
+   /* For environments using 16-bit integers OUTBUFSIZ must be limited to
+    * less than 64k (do_string() uses "unsigned" in calculations involving
+    * OUTBUFSIZ).  This is achieved by defining MED_MEM when WSIZE = 64k (aka
+    * Deflate64 support enabled) or EOL markers consist multiple characters.
+    * (The rule gets applied AFTER the default rule for INBUFSIZ because it
+    * is not neccessary to reduce INBUFSIZE in this case.)
+    */
+#  if (!defined(SMALL_MEM) && !defined(MED_MEM))
+#    define MED_MEM
+#  endif
+#endif
+
 /* Logic for case of small memory, length of EOL > 1:  if OUTBUFSIZ == 2048,
  * OUTBUFSIZ>>1 == 1024 and OUTBUFSIZ>>7 == 16; therefore rawbuf is 1008 bytes
  * and transbuf 1040 bytes.  Have room for 32 extra EOL chars; 1008/32 == 31.5
@@ -892,7 +960,19 @@
 #  define LoadFarStringSmall2(x) fLoadFarStringSmall2(__G__ (x))
 #  if (defined(_MSC_VER) && (_MSC_VER >= 600))
 #    define zfstrcpy(dest, src)  _fstrcpy((dest), (src))
+#    define zfstrcmp(s1, s2)     _fstrcmp((s1), (s2))
 #  endif
+#  if !(defined(SFX) || defined(FUNZIP))
+#    if (defined(_MSC_VER))
+#      define zfmalloc(sz)       _fmalloc((sz))
+#      define zffree(x)          _ffree(x)
+#    endif
+#    if (defined(__TURBOC__))
+#      include <alloc.h>
+#      define zfmalloc(sz)       farmalloc((unsigned long)(sz))
+#      define zffree(x)          farfree(x)
+#    endif
+#  endif /* !(SFX || FUNZIP) */
 #  ifndef Far
 #    define Far far  /* __far only works for MSC 6.00, not 6.0a or Borland */
 #  endif
@@ -906,6 +986,9 @@
    typedef short  shrint;            /* short/int or "shrink int" (unshrink) */
 #else
 #  define zfstrcpy(dest, src)       strcpy((dest), (src))
+#  define zfstrcmp(s1, s2)          strcmp((s1), (s2))
+#  define zfmalloc                  malloc
+#  define zffree(x)                 free(x)
 #  ifdef QDOS
 #    define LoadFarString(x)        Qstrfix(x)   /* fix up _ for '.' */
 #    define LoadFarStringSmall(x)   Qstrfix(x)
@@ -935,6 +1018,10 @@
 #  define Far
 #endif
 
+#ifndef Cdecl
+#  define Cdecl
+#endif
+
 #ifndef MAIN
 #  define MAIN  main
 #endif
@@ -974,8 +1061,11 @@
 #  define COPYRIGHT_CLEAN
 #endif
 
+/* The LZW patent is expired worldwide since 2004-Jul-07, so USE_UNSHRINK
+ * is now enabled by default.  See unshrink.c.
+ */
 #if (!defined(LZW_CLEAN) && !defined(USE_UNSHRINK))
-#  define LZW_CLEAN
+#  define USE_UNSHRINK
 #endif
 
 #ifndef O_BINARY
@@ -1110,9 +1200,13 @@
 
 #if (defined(MALLOC_WORK) && !defined(MY_ZCALLOC))
    /* Any system without a special calloc function */
+# ifndef zcalloc
 #  define zcalloc(items, size) \
           (zvoid far *)calloc((unsigned)(items), (unsigned)(size))
+# endif
+# ifndef zcfree
 #  define zcfree    free
+# endif
 #endif /* MALLOC_WORK && !MY_ZCALLOC */
 
 #ifdef REGULUS  /* returns the inode number on success(!)...argh argh argh */
@@ -1224,22 +1318,22 @@
 #endif
 
 #define CENTRAL_HDR_SIG   "\001\002"   /* the infamous "PK" signature bytes, */
-#define LOCAL_HDR_SIG     "\003\004"   /*  sans "PK" (so unzip executable not */
+#define LOCAL_HDR_SIG     "\003\004"   /*  w/o "PK" (so unzip executable not */
 #define END_CENTRAL_SIG   "\005\006"   /*  mistaken for zipfile itself) */
 #define EXTD_LOCAL_SIG    "\007\010"   /* [ASCII "\113" == EBCDIC "\080" ??] */
 
 /** internal-only return codes **/
 #define IZ_DIR            76   /* potential zipfile is a directory */
 /* special return codes for mapname() */
-#define MPN_OK            0     /* mapname successful */
-#define MPN_INF_TRUNC     1<<8  /* caution - filename truncated */
-#define MPN_INF_SKIP      2<<8  /* info  - skipped because nothing to do */
-#define MPN_ERR_SKIP      3<<8  /* error - entry skipped */
-#define MPN_ERR_TOOLONG   4<<8  /* error - path too long */
-#define MPN_NOMEM         10<<8 /* error - out of memory, file skipped */
-#define MPN_CREATED_DIR   16<<8 /* directory created: set time & permissions */
-#define MPN_VOL_LABEL     17<<8 /* volume label, but can't set on hard disk */
-#define MPN_INVALID       99<<8 /* internal logic error, should never reach */
+#define MPN_OK            0      /* mapname successful */
+#define MPN_INF_TRUNC    (1<<8)  /* caution - filename truncated */
+#define MPN_INF_SKIP     (2<<8)  /* info  - skipped because nothing to do */
+#define MPN_ERR_SKIP     (3<<8)  /* error - entry skipped */
+#define MPN_ERR_TOOLONG  (4<<8)  /* error - path too long */
+#define MPN_NOMEM        (10<<8) /* error - out of memory, file skipped */
+#define MPN_CREATED_DIR  (16<<8) /* directory created: set time & permission */
+#define MPN_VOL_LABEL    (17<<8) /* volume label, but can't set on hard disk */
+#define MPN_INVALID      (99<<8) /* internal logic error, should never reach */
 /* mask for internal mapname&checkdir return codes */
 #define MPN_MASK          0x7F00
 /* error code for extracting/testing extra field blocks */
@@ -1301,7 +1395,10 @@
 #define BEOS_             16   /* hybrid POSIX/database filesystem */
 #define TANDEM_           17   /* Tandem NSK */
 #define THEOS_            18   /* THEOS */
-#define NUM_HOSTS         19   /* index of last system + 1 */
+#define MAC_OSX_          19   /* Mac OS/X (Darwin) */
+#define ATHEOS_           30   /* AtheOS */
+#define NUM_HOSTS         31   /* index of last system + 1 */
+/* don't forget to update zipinfo.c appropiately if NUM_HOSTS changes! */
 
 #define STORED            0    /* compression methods */
 #define SHRUNK            1
@@ -1314,7 +1411,9 @@
 #define DEFLATED          8
 #define ENHDEFLATED       9
 #define DCLIMPLODED      10
-#define NUM_METHODS      11    /* index of last method + 1 */
+#define PKRESMOD11       11
+#define BZIP2ED          12
+#define NUM_METHODS      13    /* index of last method + 1 */
 /* don't forget to update list_files(), extract.c and zipinfo.c appropriately
  * if NUM_METHODS changes */
 
@@ -1347,6 +1446,7 @@
 #define EF_MVS       0x470f    /* Info-ZIP's MVS ("\017G") */
 #define EF_ACL       0x4c41    /* (OS/2) access control list ("AL") */
 #define EF_NTSD      0x4453    /* NT security descriptor ("SD") */
+#define EF_ATHEOS    0x7441    /* AtheOS ("At") */
 #define EF_BEOS      0x6542    /* BeOS ("Be") */
 #define EF_QDOS      0xfb4a    /* SMS/QDOS ("J\373") */
 #define EF_AOSVS     0x5356    /* AOS/VS ("VS") */
@@ -1385,8 +1485,8 @@
 #define EB_FLGS_OFFS      4    /* offset of flags area in generic compressed
                                   extra field blocks (BEOS, MAC, and others) */
 #define EB_OS2_HLEN       4    /* size of OS2/ACL compressed data header */
-#define EB_BEOS_HLEN      5    /* length of BeOS e.f attribute header */
-#define EB_BE_FL_UNCMPR   0x01 /* "BeOS attributes uncompressed" bit flag */
+#define EB_BEOS_HLEN      5    /* length of BeOS&AtheOS e.f attribute header */
+#define EB_BE_FL_UNCMPR   0x01 /* "BeOS&AtheOS attribs uncompr." bit flag */
 #define EB_MAC3_HLEN      14   /* length of Mac3 attribute block header */
 #define EB_SMARTZIP_HLEN  64   /* fixed length of the SmartZip extra field */
 #define EB_M3_FL_DATFRK   0x01 /* "this entry is data fork" flag */
@@ -1466,6 +1566,17 @@
 #  define QCOND2    (!uO.qflag)
 #endif
 
+#ifdef WILD_STOP_AT_DIR
+#  define __WDLPRO  , int sepc
+#  define __WDL     , sepc
+#  define __WDLDEF  int sepc;
+#  define WISEP     , (uO.W_flag ? '/' : '\0')
+#else
+#  define __WDLPRO
+#  define __WDL
+#  define __WDLDEF
+#  define WISEP
+#endif
 
 
 
@@ -1500,24 +1611,29 @@ typedef struct iztimes {
 } iztimes;
 
 #ifdef SET_DIR_ATTRIB
-   typedef struct dirtime {  /* temporary struct for holding directory info */
-       struct dirtime *next; /*  until can be sorted and set at end */
-       char *fn;             /* filename of directory */
-       union {
-           iztimes t3;       /* mtime, atime, ctime */
-           ztimbuf t2;       /* modtime, actime */
-       } u;
-       unsigned perms;       /* same as min_info.file_attr */
-       int have_uidgid;      /* flag */
-       ush uidgid[2];
-   } dirtime;
+   typedef struct direntry {    /* head of system-specific struct holding */
+       struct direntry *next;   /*  defered directory attributes info */
+       char *fn;                /* filename of directory */
+       char buf[1];             /* start of system-specific internal data */
+   } direntry;
 #endif /* SET_DIR_ATTRIB */
 
+#ifdef SYMLINKS
+   typedef struct slinkentry {  /* info for deferred symlink creation */
+       struct slinkentry *next; /* pointer to next entry in chain */
+       extent targetlen;        /* length of target filespec */
+       extent attriblen;        /* length of system-specific attrib data */
+       char *target;            /* pointer to target filespec */
+       char *fname;             /* pointer to name of link */
+       char buf[2];             /* name of link, allocs space for 2 '\0's */
+   } slinkentry;
+#endif /* SYMLINKS */
+
 typedef struct min_info {
-    long offset;
-    ulg crc;                 /* crc (needed if extended header) */
+    Z_OFF_T offset;
     ulg compr_size;          /* compressed size (needed if extended header) */
     ulg uncompr_size;        /* uncompressed size (needed if extended header) */
+    ulg crc;                 /* crc (needed if extended header) */
     ush diskstart;           /* no of volume where this entry starts */
     uch hostver;
     uch hostnum;
@@ -1529,6 +1645,9 @@ typedef struct min_info {
     unsigned lcflag : 1;     /* convert filename to lowercase */
     unsigned vollabel : 1;   /* "file" is an MS-DOS volume (disk) label */
     unsigned HasUxAtt : 1;   /* crec ext_file_attr has Unix style mode bits */
+#ifndef SFX
+    char Far *cfilname;      /* central header version of filename */
+#endif
 } min_info;
 
 typedef struct VMStimbuf {
@@ -1615,14 +1734,23 @@ typedef struct VMStimbuf {
 #      define ZIPFILE_COMMENT_LENGTH            20
 
 
+/* The following structs are used to hold all header data of a zip entry.
+   Traditionally, the structs' layouts followed the data layout of the
+   corresponding zipfile header structures.  However, the zipfile header
+   layouts were designed in the old ages of 16-bit CPUs, they are subject
+   to structure padding and/or alignment issues on newer systems with a
+   "natural word width" of more than 2 bytes.
+   Please note that the structure members are now reordered by size
+   (top-down), to prevent internal padding and optimize memory usage!
+ */
    typedef struct local_file_header {                 /* LOCAL */
+       ulg csize;
+       ulg ucsize;
+       ulg last_mod_dos_datetime;
+       ulg crc32;
        uch version_needed_to_extract[2];
        ush general_purpose_bit_flag;
        ush compression_method;
-       ulg last_mod_dos_datetime;
-       ulg crc32;
-       ulg csize;
-       ulg ucsize;
        ush filename_length;
        ush extra_field_length;
    } local_file_hdr;
@@ -1775,7 +1903,7 @@ void     defer_leftover_input OF((__GPRO));
 unsigned readbuf              OF((__GPRO__ char *buf, register unsigned len));
 int      readbyte             OF((__GPRO));
 int      fillinbuf            OF((__GPRO));
-int      seek_zipf            OF((__GPRO__ LONGINT abs_offset));
+int      seek_zipf            OF((__GPRO__ Z_OFF_T abs_offset));
 #ifdef FUNZIP
    int   flush                OF((__GPRO__ ulg size));  /* actually funzip.c */
 #else
@@ -1822,6 +1950,9 @@ ulg      makelong             OF((ZCONST uch *sig));
    #ifndef zfstrcpy
      char Far * Far zfstrcpy  OF((char Far *s1, const char Far *s2));
    #endif
+   #if (!defined(SFX) && !defined(zfstrcmp))
+     int Far zfstrcmp         OF((const char Far *s1, const char Far *s2));
+   #endif
 #endif
 
 
@@ -1855,7 +1986,7 @@ int    explode                   OF((__GPRO));                  /* explode.c */
 int    huft_free                 OF((struct huft *t));          /* inflate.c */
 int    huft_build                OF((__GPRO__ ZCONST unsigned *b, unsigned n,
                                      unsigned s, ZCONST ush *d, ZCONST uch *e,
-                                     struct huft **t, int *m));
+                                     struct huft **t, unsigned *m));
 #ifdef USE_ZLIB
    int    UZinflate              OF((__GPRO__ int is_defl64));  /* inflate.c */
 #  define inflate_free(x)        inflateEnd(&((Uz_Globs *)(&G))->dstrm)
@@ -1890,7 +2021,7 @@ int    huft_build                OF((__GPRO__ ZCONST unsigned *b, unsigned n,
    /* this obsolescent entry point kept for compatibility: */
    int      UzpUnzip              OF((int argc, char **argv));/* use UzpMain */
 #ifdef OS2DLL
-   int      varmessage            OF((__GPRO__ uch *buf, ulg size));
+   int      varmessage            OF((__GPRO__ ZCONST uch *buf, ulg size));
    int      varputchar            OF((__GPRO__ int c));         /* rexxapi.c */
    int      finish_REXX_redirect  OF((__GPRO));                 /* rexxapi.c */
 #endif
@@ -2020,7 +2151,7 @@ int      envargs         OF((int *Pargc, char ***Pargv,
 void     mksargs         OF((int *argcp, char ***argvp));       /* envargs.c */
 
 int      match           OF((ZCONST char *s, ZCONST char *p,
-                             int ic));                            /* match.c */
+                             int ic __WDLPRO));                   /* match.c */
 int      iswild          OF((ZCONST char *p));                    /* match.c */
 
 #ifdef DYNALLOC_CRCTAB
@@ -2031,7 +2162,8 @@ int      iswild          OF((ZCONST char *p));                    /* match.c */
    ulg      crc32           OF((ulg crc, ZCONST uch *buf, extent len));
 #endif /* !USE_ZLIB */                        /* assembler source or crc32.c */
 
-int      dateformat      OF((void));              /* currently, only msdos.c */
+int      dateformat      OF((void));                                /* local */
+char     dateseparator   OF((void));                                /* local */
 #ifndef WINDLL
    void  version         OF((__GPRO));                              /* local */
 #endif
@@ -2040,20 +2172,24 @@ int      mapname         OF((__GPRO__ int renamed));                /* local */
 int      checkdir        OF((__GPRO__ char *pathcomp, int flag));   /* local */
 char    *do_wild         OF((__GPRO__ ZCONST char *wildzipfn));     /* local */
 char    *GetLoadPath     OF((__GPRO));                              /* local */
-#if (defined(MORE) && (defined(BEO_UNX) || defined(QDOS) || defined(VMS)))
+#if (defined(MORE) && (defined(ATH_BEO_UNX) || defined(QDOS) || defined(VMS)))
    int screensize        OF((int *tt_rows, int *tt_cols));          /* local */
 # if defined(VMS)
    int screenlinewrap    OF((void));                                /* local */
 # endif
-#endif /* MORE && (BEO_UNX || QDOS || VMS) */
+#endif /* MORE && (ATH_BEO_UNX || QDOS || VMS) */
 #ifdef OS2_W32
    int   SetFileSize     OF((FILE *file, ulg filesize));            /* local */
 #endif
 #ifndef MTS /* macro in MTS */
    void  close_outfile   OF((__GPRO));                              /* local */
 #endif
+#ifdef SET_SYMLINK_ATTRIBS
+   int  set_symlnk_attribs  OF((__GPRO__ slinkentry *slnk_entry));  /* local */
+#endif
 #ifdef SET_DIR_ATTRIB
-   int   set_direc_attribs  OF((__GPRO__ dirtime *d));              /* local */
+   int   defer_dir_attribs  OF((__GPRO__ direntry **pd));           /* local */
+   int   set_direc_attribs  OF((__GPRO__ direntry *d));             /* local */
 #endif
 #ifdef TIMESTAMP
 # ifdef WIN32
@@ -2070,6 +2206,9 @@ char    *GetLoadPath     OF((__GPRO));                              /* local */
 #ifdef SYSTEM_SPECIFIC_CTOR
    void  SYSTEM_SPECIFIC_CTOR   OF((__GPRO));                       /* local */
 #endif
+#ifdef SYSTEM_SPECIFIC_DTOR
+   void  SYSTEM_SPECIFIC_DTOR   OF((__GPRO));                       /* local */
+#endif
 
 
 
@@ -2147,7 +2286,7 @@ char    *GetLoadPath     OF((__GPRO));                              /* local */
 #  endif
 #endif /* !Info */
 
-/*  The following macros wrappers around the fnfilter function are used many
+/*  The following macro wrappers around the fnfilter function are used many
  *  times to prepare archive entry names or name components for displaying
  *  listings and (warning/error) messages. They use sections in the upper half
  *  of 'slide' as buffer, since their output is normally fed through the
@@ -2196,14 +2335,14 @@ char    *GetLoadPath     OF((__GPRO));                              /* local */
 #else
 #  define FLUSH(w)  ((G.mem_mode) ? memflush(__G__ redirSlide,(ulg)(w)) \
                                   : flush(__G__ redirSlide,(ulg)(w),0))
-#  define NEXTBYTE  (--G.incnt >= 0 ? (int)(*G.inptr++) : readbyte(__G))
+#  define NEXTBYTE  (G.incnt-- > 0 ? (int)(*G.inptr++) : readbyte(__G))
 #endif
 
 
 #define READBITS(nbits,zdest) {if(nbits>G.bits_left) {int temp; G.zipeof=1;\
   while (G.bits_left<=8*(int)(sizeof(G.bitbuf)-1) && (temp=NEXTBYTE)!=EOF) {\
   G.bitbuf|=(ulg)temp<<G.bits_left; G.bits_left+=8; G.zipeof=0;}}\
-  zdest=(shrint)((ush)G.bitbuf&mask_bits[nbits]);G.bitbuf>>=nbits;\
+  zdest=(shrint)((unsigned)G.bitbuf&mask_bits[nbits]);G.bitbuf>>=nbits;\
   G.bits_left-=nbits;}
 
 /*
@@ -2220,7 +2359,7 @@ char    *GetLoadPath     OF((__GPRO));                              /* local */
  *              G.zipeof = 0;
  *          }
  *      }
- *      zdest = (shrint)((ush)G.bitbuf & mask_bits[nbits]);
+ *      zdest = (shrint)((unsigned)G.bitbuf & mask_bits[nbits]);
  *      G.bitbuf >>= nbits;
  *      G.bits_left -= nbits;
  *  }
@@ -2437,7 +2576,7 @@ char    *GetLoadPath     OF((__GPRO));                              /* local */
 /*  Global constants  */
 /**********************/
 
-   extern ZCONST ush near  mask_bits[];
+   extern ZCONST unsigned near mask_bits[17];
    extern ZCONST char *fnames[2];
 
 #ifdef EBCDIC
